@@ -155,11 +155,23 @@ def data_service(input, output, http_request):
 
 class StreamApi(tornado.web.RequestHandler):
 
-    async def post(self, action, *args, **kwargs):
+    async def post(self, path, *args, **kwargs):
+
+        if '/' in path:
+            i = path.find('/')
+            action = path[:i]
+            path_arg = path[i+1:]
+        else:
+            action = path
+            path_arg = None
 
         input = {
             'action': action,
-            'arg': self.request.body,
+            'args': {
+                'stream': self.request.body,
+                'path_arg': path_arg,
+                'http_request': self.request,
+            }
         }
         output = {
             'result': {},
@@ -167,7 +179,7 @@ class StreamApi(tornado.web.RequestHandler):
         }
         thread = threading.Thread(
             target=stream_service,
-            args=(input, output, self.request)
+            args=(input, output)
         )
         thread.setDaemon(True)
         thread.start()
@@ -200,10 +212,10 @@ class StreamApi(tornado.web.RequestHandler):
         self.set_header('Access-Control-Allow-Methods', 'GET, POST, OPTIONS')
 
 
-def stream_service(input, output, http_request):
+def stream_service(input, output):
 
     action = input.get('action')
-    arg = input.get('arg')
+    args = input.get('args')
 
     public_actions = getattr(AppCore, 'public_actions', None)
     if public_actions is not None:
@@ -229,7 +241,7 @@ def stream_service(input, output, http_request):
 
     result = {}
     try:
-        result['value'] = action_obj(arg, http_request)
+        result['value'] = action_obj(**args)
         result['status'] = 'ok'
     except Exception as e:
         logging.error(str(e))
@@ -242,7 +254,15 @@ def stream_service(input, output, http_request):
 
 class FileApi(tornado.web.RequestHandler):
 
-    async def post(self, action, *args, **kwargs):
+    async def post(self, path, *args, **kwargs):
+
+        if '/' in path:
+            i = path.find('/')
+            action = path[:i]
+            path_arg = path[i+1:]
+        else:
+            action = path
+            path_arg = None
         
         try:
             file_obj = self.request.files.get(action)[0]
@@ -261,7 +281,11 @@ class FileApi(tornado.web.RequestHandler):
 
         input = {
             'action': action,
-            'arg': file_body,
+            'args': {
+                'stream': file_body,
+                'path_arg': path_arg,
+                'http_request': self.request,
+            }
         }
         output = {
             'result': {},
@@ -269,7 +293,7 @@ class FileApi(tornado.web.RequestHandler):
         }
         thread = threading.Thread(
             target=stream_service,
-            args=(input, output, self.request)
+            args=(input, output)
         )
         thread.setDaemon(True)
         thread.start()
@@ -324,8 +348,8 @@ class GatewayApi(tornado.web.RequestHandler):
         result = output['result']
 
         if result['status'] == 'ok':
-            new_access_token = result['value']['new_access_token']
-            new_refresh_token = result['value']['new_refresh_token']
+            new_access_token = result['value'].get('new_access_token')
+            new_refresh_token = result['value'].get('new_refresh_token')
             response = result['value']['response']
             if new_access_token is not None and new_refresh_token is not None:
                 self.set_cookie('access_token', new_access_token)
@@ -641,8 +665,8 @@ def start():
             (r'/management/health', HealthChecker),
             (r'/special/(.*)', SpecialApi),
             (r'/api/data', DataApi),
-            (r'/api/stream/(\w+)', StreamApi),
-            (r'/api/file/(\w+)', FileApi),
+            (r'/api/stream/(.*)', StreamApi),
+            (r'/api/file/(.*)', FileApi),
             (r'/(.*)', tornado.web.StaticFileHandler, {'path': 'webapp/www', 'default_filename': 'index.html'}),
         ]
 
